using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace PasseioDoCavalo
{
    public partial class Form1 : Form
    {
        public int[,] tabuleiro;
        public Label[,] tabuleiroLabel;
        public int ordemDoTabuleiro = 0;
        public int x0;
        public int y0;
        public int posicaoAtualX = 0;
        public int posicaoAtualY = 0;
        public int[] deslocamentoX = { 2, 1, -1, -2, -2, -1, 1, 2 };
	    public int[] deslocamentoY = { 1, 2, 2, 1, -1, -2, -2, -1 };
	    public int totalDeCelulas;
        public int tempoDeEspera;
        public long totalDeOperacoes;
        public long totalDeOperacoes2;
        int larguraLabel = 60;
	
        public Form1()
        {
            InitializeComponent();
        }


        bool jogadaEhAceitavel(int x, int y)
        {
            bool resultado = (x >= 0 && x <= ordemDoTabuleiro - 1);
            resultado = resultado && (y >= 0 && y <= ordemDoTabuleiro - 1);
            resultado = resultado && (tabuleiro[x,y] == 0);
            return resultado;
        }

        bool tentaMover(int i, int x, int y)
        {
            // Verifica a quantidade de movimentos
            bool problemaResolvido = (i > totalDeCelulas);
            int k = 0;
            int u, v;  // u,v: posicao de destino  -  x, y: posicao atual
            while (!problemaResolvido && k < 8)
            { // 8 eh o numero de movimentos teoricamente possiveis
                u = x + deslocamentoX[k];
                v = y + deslocamentoY[k];
                totalDeOperacoes2++;
                if (jogadaEhAceitavel(u, v))
                {
                    totalDeOperacoes++;

                    tabuleiro[u,v] = i;
                    tabuleiroLabel[x, y].BackColor = Color.LightBlue;
                    tabuleiroLabel[u, v].Text = i.ToString();
                    tabuleiroLabel[u, v].BackColor = Color.Green;
                    lblOperacoes.Text = totalDeOperacoes.ToString();
                    lblOperacoes2.Text = totalDeOperacoes2.ToString();
                    grpTabuleiro.Refresh();
                    System.Threading.Thread.Sleep(tempoDeEspera);

                    problemaResolvido = tentaMover(i + 1, u, v); // tenta outro movimento
                    if (!problemaResolvido)
                    {
                        totalDeOperacoes++;
                        tabuleiroLabel[u, v].BackColor = Color.Red;
                        tabuleiroLabel[x, y].BackColor = Color.Green;
                        grpTabuleiro.Refresh();
                        System.Threading.Thread.Sleep(tempoDeEspera);
                        tabuleiroLabel[u, v].Text = "0";
                        tabuleiroLabel[u, v].BackColor = Color.White;
                        grpTabuleiro.Refresh();
                        System.Threading.Thread.Sleep(tempoDeEspera);

                        tabuleiro[u,v] = 0; // no sucedido. Descarta movimento
                    }
                }
                k = k + 1;
            }
            return problemaResolvido;
        }

        void resolverAtumoticamente(int x, int y)
        {
            bool problemaResolvido = tentaMover(2, x, y);
            lblOperacoes.Text = totalDeOperacoes.ToString();
            lblOperacoes2.Text = totalDeOperacoes2.ToString();
            if (problemaResolvido)
            {
                /*
                System.out.println("Passeio resolvido:");
                for (int i = 0; i < ordemDoTabuleiro; i++) {
                    for (int j = 0; j < ordemDoTabuleiro; j++) {
                        if (tabuleiro[i,j]<100) System.out.print(" "); // apenas para deixar a impressao mais organizada 
                        if (tabuleiro[i,j]<10) System.out.print(" "); // apenas para deixar a impressao mais organizada
                        System.out.print(tabuleiro[i,j] + " ");
                    }
                    System.out.println();
                }*/
            }
            else
            {
                MessageBox.Show("Passeio impossivel de ser realizado.");
            }
            
        }


        private void button1_Click(object sender, EventArgs e)
        {
            iniciarTabuleiro();
            resolverAtumoticamente(x0, y0);
        }

        private void iniciarTabuleiro()
        {

            //Limpando tabuleiro
            for (int x = 0; x < ordemDoTabuleiro; x++)
            {
                for (int y = 0; y < ordemDoTabuleiro; y++)
                {
                    grpTabuleiro.Controls.Remove(tabuleiroLabel[x, y]);
                }
            }

            x0 = 0;
            y0 = 0;
            totalDeOperacoes = 0;
            ordemDoTabuleiro = 4;
            tempoDeEspera = 5;
            try
            {
                ordemDoTabuleiro = int.Parse(cbTamanho.Text);
            }
            catch
            {
                ordemDoTabuleiro = 4;
            }
            if ((ordemDoTabuleiro < 0) || (ordemDoTabuleiro > 8)) ordemDoTabuleiro = 4;

            try
            {
                tempoDeEspera = int.Parse(cbAtraso.Text);
            }
            catch
            {
                tempoDeEspera = 5;
            }

            try
            {
                x0 = int.Parse(cbX0.Text);
            }
            catch
            {
                x0 = 0;
            }
            if ((x0 < 0) || (x0 > ordemDoTabuleiro-1)) x0=0;

            try
            {
                y0 = int.Parse(cbY0.Text);
            }
            catch
            {
                y0 = 0;
            }
            if ((y0 < 0) || (y0 > ordemDoTabuleiro - 1)) y0 = 0;

            posicaoAtualX = x0;
            posicaoAtualY = y0;

            totalDeCelulas = ordemDoTabuleiro * ordemDoTabuleiro;
            int tamanho = ordemDoTabuleiro;
            tabuleiro = new int[tamanho, tamanho];
            tabuleiroLabel = new Label[tamanho, tamanho];


            for (int i = 0; i < tamanho; i++)
            {
                for (int j = 0; j < tamanho; j++)
                {
                    tabuleiro[j, i] = 0;
                    Label lbl1 = new Label();
                    lbl1.Name = "l1";
                    lbl1.TextAlign = ContentAlignment.MiddleCenter;
                    lbl1.Font = lblTitulo.Font;
                    int id = i + tamanho * j;
                    lbl1.Text = "0";
                    lbl1.Visible = true;
                    lbl1.AutoSize = false;
                    lbl1.Width = larguraLabel;
                    lbl1.Height = larguraLabel;
                    lbl1.BackColor = Color.White;
                    lbl1.Location = new System.Drawing.Point(i * (larguraLabel + 1) + 50, j * (larguraLabel + 1) + 50);
                    lbl1.Click += new System.EventHandler(cliqueDoUsuario);
                    tabuleiroLabel[j, i] = lbl1;
                    grpTabuleiro.Controls.Add(lbl1);
                }
            }
            totalDeOperacoes = 0;
            totalDeOperacoes2 = 0;
            tabuleiro[x0, y0] = 1;
            tabuleiroLabel[x0, y0].Text = "1";
            tabuleiroLabel[x0, y0].BackColor = Color.Green;

        }

        private void cliqueDoUsuario(object sender, EventArgs e)
        {
            Label clicado = ((Label)sender);
            Label anterior = tabuleiroLabel[posicaoAtualX,posicaoAtualY];
            int distY = (clicado.Location.X - anterior.Location.X) / (larguraLabel + 1);
            int distX = (clicado.Location.Y - anterior.Location.Y) / (larguraLabel + 1);

            int valorAnterior = int.Parse(anterior.Text);
            if (clicado.Text != "0")
            {
                int valorClicado = int.Parse(clicado.Text);

                if (valorClicado == valorAnterior - 1)
                {  // desfazer ultima jogada
                    clicado.BackColor = Color.Green;
                    posicaoAtualX += distX;
                    posicaoAtualY += distY;
                    anterior.BackColor = Color.White;
                    anterior.Text = "0";
                }
                else
                {
                    MessageBox.Show("Jogada invlida: posio de destino j foi visitada.");
                }
            }
            else
            {
                if ((Math.Abs(distX) == 2 && Math.Abs(distY) == 1) || ((Math.Abs(distY) == 2 && Math.Abs(distX) == 1)))
                {
                    clicado.BackColor = Color.Green;
                    posicaoAtualX += distX;
                    posicaoAtualY += distY;
                    anterior.BackColor = Color.LightBlue;
                    int valorClicado = valorAnterior + 1;
                    clicado.Text = valorClicado.ToString();
                }
                else
                {
                    MessageBox.Show("Jogada invlida: o movimento sugerido no  um movimento vlido para um cavalo.");
                }
            }
            grpTabuleiro.Refresh(); 
        }

        private void btnTabuleiro_Click(object sender, EventArgs e)
        {
            iniciarTabuleiro();
        }

        private void cbTamanho_SelectedValueChanged(object sender, EventArgs e)
        {
            int ordemTemp = 4;
            try
            {
                ordemTemp = int.Parse(cbTamanho.Text);
            }
            catch
            {
                ordemTemp = 4;
            }
            if ((ordemTemp < 0) || (ordemTemp > 8)) ordemTemp = 4;
            cbX0.Items.Clear();
            cbY0.Items.Clear();
            for (int i = 0; i < ordemTemp; i++)
            {
                cbX0.Items.Add(i.ToString());
                cbY0.Items.Add(i.ToString());
            }
            cbX0.Text = "0";
            cbY0.Text = "0";
            iniciarTabuleiro();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            iniciarTabuleiro();
        }

        private void cbY0_SelectedIndexChanged(object sender, EventArgs e)
        {
            iniciarTabuleiro();
        }

        private void cbX0_SelectedIndexChanged(object sender, EventArgs e)
        {
            iniciarTabuleiro();
        }
    }
}